<?php

/**
 * Core class to add new feature to WordPress Rewrite API (such as start points)
 *
 */
class Penci_AMP_Rewrite_Rules {

	public $startpints = array();

	protected $exclude_extra_permastructs = array(
		'category'          => true,
		'post_tag'          => true,
		'post_format'       => true,

		'product_variation' => true,
		'shop_order_refund' => true,

		'vc_grid_item'      => true,
	);

	/**
	 * Penci_AMP_Rewrite_Rules constructor.
	 */
	public function __construct() {
		$this->add_rewrite_rules_hooks();
	}

	public function add_rewrite_rules_hooks() {

		$hooks = array(
			'post_rewrite_rules',
			'date_rewrite_rules',
			'root_rewrite_rules',
			'comments_rewrite_rules',
			'search_rewrite_rules',
			'author_rewrite_rules',
			'page_rewrite_rules',
			'category_rewrite_rules',
			'post_tag_rewrite_rules',
			'post_format_rewrite_rules',
		);

		foreach ( $hooks as $hook ) {
			add_filter( $hook, array( $this, 'generate_rewrite_rules' ), 9999 );
		}

		add_action( 'root_rewrite_rules', array( $this, 'register_extra_permastruct_hooks' ) );
	}

	public function register_extra_permastruct_hooks( $rules ) {
		global $wp_rewrite;

		$extra_permastruct = array_diff_key( $wp_rewrite->extra_permastructs, $this->get_exclude_extra_permastructs() );

		foreach ( $extra_permastruct as $permastructname => $struct ) {

			if ( empty( $struct['walk_dirs'] ) ) {
				continue;
			}

			if ( ! has_filter( "{$permastructname}_rewrite_rules", array( $this, 'generate_rewrite_rules' ) ) ) {

				add_filter( "{$permastructname}_rewrite_rules", array( $this, 'generate_rewrite_rules' ), 9999 );
			}
		}

		return $rules;
	}

	public function get_exclude_extra_permastructs() {
		return $this->exclude_extra_permastructs;
	}

	public function set_exclude_extra_permastructs( $permastructname ) {

		foreach ( (array) $permastructname as $name ) {
			$this->exclude_extra_permastructs[ $name ] = true;
		}
	}


	public function flush_exclude_extra_permastructs() {

		$this->exclude_extra_permastructs = array();

		return true;
	}


	protected function get_current_sp_mask() {
		global $wp_rewrite;

		$current_filter = current_filter();

		switch ( $current_filter ) {

			case 'post_rewrite_rules':
				$ep_mask = EP_PERMALINK;
				break;

			case 'date_rewrite_rules':
				$ep_mask = EP_DATE;
				break;

			case 'root_rewrite_rules':
				$ep_mask = EP_ROOT;
				break;

			case 'comments_rewrite_rules':
				$ep_mask = EP_COMMENTS;
				break;

			case 'search_rewrite_rules':
				$ep_mask = EP_SEARCH;
				break;

			case 'author_rewrite_rules':
				$ep_mask = EP_AUTHORS;
				break;

			case 'page_rewrite_rules':
				$ep_mask = EP_PAGES;
				break;

			case 'category_rewrite_rules':
				$ep_mask = EP_CATEGORIES;
				break;

			case 'post_tag_rewrite_rules':
				$ep_mask = EP_TAGS;
				break;

			default:

				$ep_mask = EP_NONE;

				if ( preg_match( '/(.+)_rewrite_rules$/', $current_filter, $matched ) ) {

					if ( isset( $wp_rewrite->extra_permastructs[ $matched[1] ]['ep_mask'] ) ) {

						$ep_mask = max(
							$wp_rewrite->extra_permastructs[ $matched[1] ]['ep_mask'],
							1
						);
					}
				}
		}


		return $ep_mask;
	}


	public function get_start_points() {

		static $query_append;

		if ( is_null( $query_append ) ) {
			$query_append = array();

			foreach ( (array) $this->startpints as $endpoint ) {
				if ( $endpoint[3] ) {
					$spmatch = $endpoint[1] . '/';
				} else {
					$spmatch = $endpoint[1] . '/([^/]+)?/?';
				}

				$epquery                  = '&' . $endpoint[2] . '=';
				$query_append[ $spmatch ] = array( $endpoint[0], $epquery, $endpoint[3] );
			}
		}

		return $query_append;
	}

	protected function _increase_preg_index_replace_callback( $matched ) {

		global $wp_rewrite;

		$index = intval( $matched[1] );

		return $wp_rewrite->preg_index( $index + 1 );
	}

	public function generate_rewrite_rules( $rewrite_rules ) {

		global $wp_rewrite;

		$current_SP = $this->get_current_sp_mask();

		$startpints = $this->get_start_points();

		if ( ! $startpints ) {
			return $rewrite_rules;
		}

		$results = array();

		foreach ( $rewrite_rules as $regex => $query ) {

			$vars = array();

			wp_parse_str( $query, $vars );

			if ( ! isset( $vars['feed'] ) ) {
				foreach ( $startpints as $spregex => $sp ) {
					if ( $sp[0] & $current_SP ) {

						if ( $sp[2] ) {
							$startpint_query = $query . $sp[1] . '1';
						} else {
							$startpint_query = $this->increase_pattern_preg_index( $query ) . $sp[1] . $wp_rewrite->preg_index( 1 );
						}

						$results[ $spregex . $regex ] = $startpint_query;
					}

				}
			}

			$results[ $regex ] = $query;
		}

		return $results;
	}

	protected function increase_pattern_preg_index( $query ) {

		global $wp_rewrite;

		$pattern = preg_quote( $wp_rewrite->preg_index( 'PLACEHOLDER' ) );
		$pattern = '/' . str_replace( 'PLACEHOLDER', '(\\d+)', $pattern ) . '/';

		$query = preg_replace_callback( $pattern, array( $this, '_increase_preg_index_replace_callback' ), $query );

		return $query;
	}

	public function add_startpint( $name, $places, $query_var = true, $single_match = true ) {

		global $wp;

		// For backward compatibility, if null has explicitly been passed as `$query_var`, assume `true`.
		if ( true === $query_var || null === func_get_arg( 2 ) ) {
			$query_var = $name;
		}

		$this->startpints[] = array( $places, $name, $query_var, $single_match );

		if ( $query_var ) {
			$wp->add_query_var( $query_var );
		}

	}

}

$GLOBALS['Penci_AMP_Rewrite_Rules'] = new Penci_AMP_Rewrite_Rules();

/**
 * Add a startpoint to rewrite rules
 */
function penci_amp_add_rewrite_startpoint( $name, $places ) {

	global $Penci_AMP_Rewrite_Rules;

	$Penci_AMP_Rewrite_Rules->add_startpint( $name, $places, true, true );
}
